@@ -44,7 +44,7 @@ use crate::keyring::{KeyEntry, KeyRing};
 use crate::light_client::LightClient;
 use crate::{config::ChainConfig, event::monitor::EventReceiver};
 
-pub(crate) mod cosmos;
+pub mod cosmos;
 pub mod counterparty;
 pub mod handle;
 pub mod runtime;
@@ -147,6 +147,8 @@ pub trait ChainEndpoint: Sized {
 
     fn get_key(&mut self) -> Result<KeyEntry, Error>;
 
+    fn add_key(&mut self, key_name: &str, key: KeyEntry) -> Result<(), Error>;
+
     // Queries
 
     fn query_commitment_prefix(&self) -> Result<CommitmentPrefix, Error>;
