@@ -125,10 +125,16 @@ pub enum ChainRequest {
         reply_to: ReplyTo<Signer>,
     },
 
-    Key {
+    GetKey {
         reply_to: ReplyTo<KeyEntry>,
     },
 
+    AddKey {
+        key_name: String,
+        key: KeyEntry,
+        reply_to: ReplyTo<()>,
+    },
+
     ModuleVersion {
         port_id: PortId,
         reply_to: ReplyTo<String>,
@@ -359,6 +365,8 @@ pub trait ChainHandle: Clone + Send + Sync + Serialize + Debug {
 
     fn get_key(&self) -> Result<KeyEntry, Error>;
 
+    fn add_key(&self, key_name: String, key: KeyEntry) -> Result<(), Error>;
+
     fn module_version(&self, port_id: &PortId) -> Result<String, Error>;
 
     fn query_status(&self) -> Result<StatusResponse, Error>;
