@@ -223,10 +223,14 @@ where
                             self.get_config(reply_to)?
                         }
 
-                        Ok(ChainRequest::Key { reply_to }) => {
+                        Ok(ChainRequest::GetKey { reply_to }) => {
                             self.get_key(reply_to)?
                         }
 
+                        Ok(ChainRequest::AddKey { key_name, key, reply_to }) => {
+                            self.add_key(key_name, key, reply_to)?
+                        }
+
                         Ok(ChainRequest::ModuleVersion { port_id, reply_to }) => {
                             self.module_version(port_id, reply_to)?
                         }
@@ -420,6 +424,16 @@ where
         reply_to.send(result).map_err(Error::send)
     }
 
+    fn add_key(
+        &mut self,
+        key_name: String,
+        key: KeyEntry,
+        reply_to: ReplyTo<()>,
+    ) -> Result<(), Error> {
+        let result = self.chain.add_key(&key_name, key);
+        reply_to.send(result).map_err(Error::send)
+    }
+
     fn module_version(&self, port_id: PortId, reply_to: ReplyTo<String>) -> Result<(), Error> {
         let result = self.chain.query_module_version(&port_id);
         reply_to.send(Ok(result)).map_err(Error::send)
