@@ -13,9 +13,8 @@ use tokio::{runtime::Runtime as TokioRuntime, sync::mpsc};
 use tracing::{debug, error, info, trace};
 
 use tendermint_rpc::{
-    event::Event as RpcEvent,
-    query::{EventType, Query},
-    Error as RpcError, SubscriptionClient, Url, WebSocketClient, WebSocketClientDriver,
+    event::Event as RpcEvent, query::Query, Error as RpcError, SubscriptionClient, Url,
+    WebSocketClient, WebSocketClientDriver,
 };
 
 use ibc::{
@@ -126,11 +125,8 @@ pub enum MonitorCmd {
 /// event handler.
 ///
 /// The default events that are queried are:
-/// - [`EventType::NewBlock`]
-/// - [`EventType::Tx`]
-///
-/// Those can be extending or overriden using
-/// [`EventMonitor::add_query`] and [`EventMonitor::set_queries`].
+/// - [`EventType::NewBlock`](tendermint_rpc::query::EventType::NewBlock)
+/// - [`EventType::Tx`](tendermint_rpc::query::EventType::Tx)
 pub struct EventMonitor {
     chain_id: ChainId,
     /// WebSocket to collect events from
@@ -155,6 +151,39 @@ pub struct EventMonitor {
     rt: Arc<TokioRuntime>,
 }
 
+// TODO: These are SDK specific, should be eventually moved.
+pub mod queries {
+    use tendermint_rpc::query::{EventType, Query};
+
+    pub fn all() -> Vec<Query> {
+        // Note: Tendermint-go supports max 5 query specifiers!
+        vec![
+            new_block(),
+            ibc_client(),
+            ibc_connection(),
+            ibc_channel(),
+            // This will be needed when we send misbehavior evidence to full node
+            // Query::eq("message.module", "evidence"),
+        ]
+    }
+
+    pub fn new_block() -> Query {
+        Query::from(EventType::NewBlock)
+    }
+
+    pub fn ibc_client() -> Query {
+        Query::eq("message.module", "ibc_client")
+    }
+
+    pub fn ibc_connection() -> Query {
+        Query::eq("message.module", "ibc_connection")
+    }
+
+    pub fn ibc_channel() -> Query {
+        Query::eq("message.module", "ibc_channel")
+    }
+}
+
 impl EventMonitor {
     /// Create an event monitor, and connect to a node
     pub fn new(
@@ -174,7 +203,7 @@ impl EventMonitor {
         let websocket_driver_handle = rt.spawn(run_driver(driver, tx_err.clone()));
 
         // TODO: move them to config file(?)
-        let event_queries = vec![Query::from(EventType::Tx), Query::from(EventType::NewBlock)];
+        let event_queries = queries::all();
 
         let monitor = Self {
             rt,
@@ -193,22 +222,6 @@ impl EventMonitor {
         Ok((monitor, rx_batch, tx_cmd))
     }
 
-    /// Set the queries to subscribe to.
-    ///
-    /// ## Note
-    /// For this change to take effect, one has to [`subscribe`] again.
-    pub fn set_queries(&mut self, queries: Vec<Query>) {
-        self.event_queries = queries;
-    }
-
-    /// Add a new query to subscribe to.
-    ///
-    /// ## Note
-    /// For this change to take effect, one has to [`subscribe`] again.
-    pub fn add_query(&mut self, query: Query) {
-        self.event_queries.push(query);
-    }
-
     /// Clear the current subscriptions, and subscribe again to all queries.
     pub fn subscribe(&mut self) -> Result<()> {
         let mut subscriptions = vec![];
@@ -360,10 +373,8 @@ impl EventMonitor {
         let rt = self.rt.clone();
 
         loop {
-            if let Ok(cmd) = self.rx_cmd.try_recv() {
-                match cmd {
-                    MonitorCmd::Shutdown => return Next::Abort,
-                }
+            if let Ok(MonitorCmd::Shutdown) = self.rx_cmd.try_recv() {
+                return Next::Abort;
             }
 
             let result = rt.block_on(async {
@@ -373,6 +384,12 @@ impl EventMonitor {
                 }
             });
 
+            // Repeat check of shutdown command here, as previous recv()
+            // may block for a long time.
+            if let Ok(MonitorCmd::Shutdown) = self.rx_cmd.try_recv() {
+                return Next::Abort;
+            }
+
             match result {
                 Ok(batch) => self.process_batch(batch).unwrap_or_else(|e| {
                     error!("[{}] {}", self.chain_id, e);
