@@ -1,6 +1,7 @@
-use std::{collections::BTreeMap, fmt};
+use alloc::collections::BTreeMap;
+use alloc::fmt;
 
-use ibc::ics24_host::identifier::ChainId;
+use ibc::core::ics24_host::identifier::ChainId;
 use itertools::Itertools;
 use serde::{Deserialize, Serialize};
 use tracing::info;
